# TestFoundry API Reference

## Table of Contents

1. [Metadata Tables](#metadata-tables)
2. [Core Functions](#core-functions)
3. [Generator Functions](#generator-functions)
4. [Random Functions](#random-functions)
5. [Utility Functions](#utility-functions)
6. [Python API](#python-api)

## Metadata Tables

### testfoundry_tb_input_field_mapping

Defines how to generate fields for input types.

```sql
CREATE TABLE testfoundry_tb_input_field_mapping (
    id INTEGER GENERATED BY DEFAULT AS IDENTITY,
    pk_input_field UUID DEFAULT gen_random_uuid() NOT NULL,
    input_type TEXT NOT NULL,
    field_name TEXT NOT NULL,
    generator_type TEXT NOT NULL,
    fk_mapping_key TEXT,
    fk_dependency_fields TEXT[],
    nested_type TEXT,
    random_function TEXT,
    required BOOLEAN DEFAULT TRUE,
    generator_group TEXT,
    group_leader BOOLEAN DEFAULT FALSE,
    group_dependency_fields TEXT[],
    field_description TEXT,
    PRIMARY KEY (input_type, field_name)
);
```

**Columns:**
- `input_type`: Name of the composite input type
- `field_name`: Name of the field within the type
- `generator_type`: One of 'random', 'resolve_fk', 'nested'
- `fk_mapping_key`: References testfoundry_tb_fk_mapping.input_type
- `fk_dependency_fields`: Fields required before this FK can be resolved
- `nested_type`: Name of nested composite type
- `random_function`: PostgreSQL function to generate random values
- `required`: Whether field must be populated
- `generator_group`: Name for grouped field generation
- `group_leader`: TRUE if this field controls the group
- `group_dependency_fields`: Fields populated by group leader

### testfoundry_tb_fk_mapping

Defines how to resolve foreign key values.

```sql
CREATE TABLE testfoundry_tb_fk_mapping (
    input_type TEXT PRIMARY KEY,
    from_expression TEXT NOT NULL,
    select_field TEXT NOT NULL,
    random_select_expression TEXT,
    random_pk_field TEXT,
    random_value_field TEXT,
    random_select_where TEXT,
    dependency_fields TEXT[],
    dependency_field_mapping JSONB,
    dependency_field_types TEXT[]
);
```

**Columns:**
- `input_type`: Logical name for the FK mapping
- `from_expression`: SQL FROM clause including JOINs
- `select_field`: Field to SELECT when resolving FK
- `random_select_expression`: Custom SELECT for complex returns
- `random_pk_field`: PK field name for dynamic JSON building
- `random_value_field`: Display value field name
- `random_select_where`: WHERE conditions
- `dependency_fields`: Input fields this FK depends on
- `dependency_field_mapping`: Maps input fields to SQL fields
- `dependency_field_types`: Field types for proper casting

### testfoundry_tb_entity_dependents

Tracks entity relationships for cascade testing.

```sql
CREATE TABLE testfoundry_tb_entity_dependents (
    entity_name TEXT NOT NULL,
    dependent_entity_name TEXT NOT NULL,
    dependency_type TEXT,
    parent_pk_fields TEXT[],
    parent_view TEXT,
    child_fk_fields TEXT[],
    child_view TEXT,
    deleted_indicator TEXT,
    deleted_value TEXT,
    archive_timestamp_field TEXT
);
```

### test_manual_scenarios

Stores custom test scenarios.

```sql
CREATE TABLE test_manual_scenarios (
    id SERIAL PRIMARY KEY,
    entity_name TEXT NOT NULL,
    scenario_type TEXT NOT NULL,
    input_json JSONB NOT NULL,
    expected_result TEXT,
    description TEXT
);
```

## Core Functions

### testfoundry_get_entity_structure

Analyzes entity metadata and structure.

```sql
testfoundry_get_entity_structure(p_entity_name TEXT)
RETURNS TABLE (
    attribute TEXT,
    value TEXT
)
```

**Parameters:**
- `p_entity_name`: Name of the entity to analyze

**Returns:**
- Table with attribute/value pairs describing entity structure

**Example:**
```sql
SELECT * FROM testfoundry_get_entity_structure('users');
```

### testfoundry_list_input_fields

Lists all fields for an input type with generation metadata.

```sql
testfoundry_list_input_fields(input_type_name TEXT)
RETURNS TABLE (
    field_name TEXT,
    field_type TEXT,
    is_required BOOLEAN,
    generator_type TEXT,
    fk_mapping_key TEXT,
    random_function TEXT,
    generator_group TEXT,
    group_leader BOOLEAN
)
```

### testfoundry_generate_random_input

Main function to generate random input data.

```sql
testfoundry_generate_random_input(
    p_input_type TEXT,
    p_debug_mode BOOLEAN DEFAULT FALSE
)
RETURNS JSONB
```

**Parameters:**
- `p_input_type`: Name of the input type to generate
- `p_debug_mode`: Enable debug output

**Returns:**
- JSONB object with generated field values

**Example:**
```sql
-- Generate random user input
SELECT testfoundry_generate_random_input('user_input');

-- With debug mode
SELECT testfoundry_generate_random_input('user_input', true);
```

## Generator Functions

### testfoundry_generate_happy_create

Generates happy path CREATE test.

```sql
testfoundry_generate_happy_create(p_entity_name TEXT)
RETURNS TEXT
```

**Returns:**
- SQL text for pgTAP test function

**Example:**
```sql
SELECT testfoundry_generate_happy_create('users');
```

### testfoundry_generate_duplicate_create

Generates test for duplicate constraint violations.

```sql
testfoundry_generate_duplicate_create(p_entity_name TEXT)
RETURNS TEXT
```

### testfoundry_generate_fk_violation_create

Generates test for foreign key violations.

```sql
testfoundry_generate_fk_violation_create(p_entity_name TEXT)
RETURNS TEXT
```

### testfoundry_generate_soft_delete

Generates test for soft delete operations.

```sql
testfoundry_generate_soft_delete(p_entity_name TEXT)
RETURNS TEXT
```

### testfoundry_generate_blocked_delete

Generates test for delete restrictions.

```sql
testfoundry_generate_blocked_delete(p_entity_name TEXT)
RETURNS TEXT
```

## Random Functions

### testfoundry_random_value

Generates random values based on PostgreSQL type.

```sql
testfoundry_random_value(p_field_type TEXT)
RETURNS TEXT
```

**Supported Types:**
- `uuid`: Random UUID
- `boolean`: Random true/false
- `integer`, `int4`, `int8`: Random integers
- `numeric`, `decimal`, `float`: Random decimals
- `timestamp`: Random timestamps
- `date`: Random dates
- `time`: Random times
- `inet`: Random IP addresses
- `macaddr`: Random MAC addresses
- Others: Random text

### testfoundry_random_value_from_mapping

Resolves foreign key using mapping definition.

```sql
testfoundry_random_value_from_mapping(
    p_fk_mapping_key TEXT,
    VARIADIC p_args TEXT[]
)
RETURNS JSONB
```

**Parameters:**
- `p_fk_mapping_key`: Key from testfoundry_tb_fk_mapping
- `p_args`: Dependency values in order

**Returns:**
- JSONB with resolved FK value(s)

### Built-in Random Generators

| Function | Returns | Example |
|----------|---------|---------|
| `testfoundry_random_email()` | Email address | user_8a3f@example.com |
| `testfoundry_random_url()` | URL | https://example.com/a3f2 |
| `testfoundry_random_phone()` | Phone number | +1-555-123-4567 |
| `testfoundry_random_latitude()` | Latitude | 42.3601 |
| `testfoundry_random_longitude()` | Longitude | -71.0589 |
| `testfoundry_random_text_from_mapping()` | Text from options | Active, Pending, Inactive |

## Utility Functions

### testfoundry_insert_entity

Inserts test entity with proper FK resolution.

```sql
testfoundry_insert_entity(
    p_entity_name TEXT,
    p_input_data JSONB
)
RETURNS UUID
```

### testfoundry_build_conditions

Builds WHERE conditions from dependencies.

```sql
testfoundry_build_conditions(
    p_dependency_fields TEXT[],
    p_dependency_mapping JSONB,
    p_args TEXT[]
)
RETURNS TEXT
```

### testfoundry_infer_dependency_field_type

Infers field types from schema.

```sql
testfoundry_infer_dependency_field_type(
    p_field_expr TEXT
)
RETURNS TEXT
```

## Python API

### FoundrySetup

Manages TestFoundry installation.

```python
class FoundrySetup:
    def __init__(self, repository: FraiseQLRepository, schema_name: str = "testfoundry")

    async def install(self) -> None:
        """Install TestFoundry schema and objects."""

    async def uninstall(self) -> None:
        """Remove TestFoundry schema."""
```

### FoundryConfig

Configuration for test generation.

```python
@dataclass
class FoundryConfig:
    schema_name: str = "testfoundry"
    test_output_dir: Path = Path("tests/generated")
    table_prefix: str = "tb_"
    view_prefix: str = "v_"
    input_type_prefix: str = "type_"
    input_type_suffix: str = "_input"
    generate_pytest: bool = True
    debug_mode: bool = False

    naming_adapters: Dict[str, str]
    test_options: Dict[str, bool]
```

### FoundryGenerator

Main test generation interface.

```python
class FoundryGenerator:
    def __init__(self, repository: FraiseQLRepository, config: FoundryConfig = None)

    async def generate_tests_for_entity(
        self,
        entity_name: str,
        table_name: str,
        input_type_name: str
    ) -> Dict[str, str]:
        """Generate all test types for an entity."""

    async def write_tests_to_files(
        self,
        tests: Dict[str, str],
        entity_name: str
    ) -> List[Path]:
        """Write tests to files."""

    async def analyze_and_populate_metadata(
        self,
        input_type: Type,
        entity_name: str,
        table_name: str
    ) -> str:
        """Analyze type and populate metadata."""
```

### FoundryAnalyzer

Analyzes types for metadata generation.

```python
class FoundryAnalyzer:
    def analyze_input_type(self, input_type: Type) -> List[FieldMapping]:
        """Analyze input type fields."""

    def analyze_entity_relationships(
        self,
        entity_name: str,
        table_name: str
    ) -> List[FKMapping]:
        """Analyze entity relationships."""

    def generate_sql_statements(self) -> str:
        """Generate SQL for metadata population."""
```

## Usage Examples

### Complete Setup Example

```python
# 1. Install TestFoundry
setup = FoundrySetup(repository)
await setup.install()

# 2. Configure
config = FoundryConfig(
    test_output_dir=Path("tests/generated"),
    generate_pytest=True,
    test_options={
        "happy_path": True,
        "constraint_violations": True,
        "fk_violations": True
    }
)

# 3. Generate tests
generator = FoundryGenerator(repository, config)
tests = await generator.generate_tests_for_entity(
    entity_name="users",
    table_name="tb_users",
    input_type_name="user_input"
)

# 4. Write to files
paths = await generator.write_tests_to_files(tests, "users")
```

### Custom Random Function

```sql
CREATE OR REPLACE FUNCTION testfoundry_random_sku()
RETURNS TEXT AS $$
BEGIN
    RETURN UPPER(
        SUBSTRING(MD5(RANDOM()::TEXT), 1, 3) || '-' ||
        LPAD((RANDOM() * 9999)::INT::TEXT, 4, '0')
    );
END;
$$ LANGUAGE plpgsql;
```

### Complex FK Mapping

```sql
INSERT INTO testfoundry_tb_fk_mapping
(input_type, from_expression, random_select_expression,
 dependency_fields, dependency_field_mapping)
VALUES
(
    'product_variant',
    '
    tb_products p
    JOIN tb_categories c ON p.category_id = c.id
    JOIN tb_variants v ON v.product_id = p.id
    ',
    'jsonb_build_object(
        ''product_id'', p.id,
        ''variant_id'', v.id,
        ''sku'', v.sku,
        ''price'', v.price
    )',
    ARRAY['category'],
    '{"category": "c.name"}'::jsonb
);
```
